//+--------------------------------------------------------------------------+
//|                                                                          |
//|                               Charting 3D                                |
//|                                                                          |
//|                         GDImage/WinLIFT project                          |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                            copyright(c) 2013                             |
//|                           www.zapsolution.com                            |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 11-01-2013 (MM-DD-YYYY)            |
//|                        Last revised : 11-23-2013 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

#include "stdafx.h"
#include <windows.h>
#include <Commctrl.h>
#include <vector>
#include <string>
#include <time.h>
#include <gl/GL.h>
#include <Uxtheme.h>

#pragma comment(lib, "..\\GDImage64.lib")
//#pragma comment(lib, "..\\GDImage32.lib")
#include "GDImage.h"

#pragma comment(lib, "..\\WinLIFT64.lib")
//#pragma comment(lib, "D:\\VS2010\\WinLIFT32.lib")
#include "WinLIFT.h"  // Search in the same directory of the current file.

#define UBOUND(T) (T.size())
#define long_proc typedef long (__stdcall *zProc)

struct ZCHART {
    long chartnumber;
    long chartseries;
    float radius;
    long listindex;
};

#define ClientW        1000
#define ClientH        560

#define IDC_CTRL       101
#define IDC_RESET      102
#define IDC_LISTBOX    103
#define IDC_STATIC     112
#define IDC_SPINNER    104

#define ID_SPIN01      1

#define GL_MULTISAMPLE_ARB 0x809D

HWND gn_GLwnd;
float gr_Percent;
ZCHART g_Chart;

long gn_ChartCaption, gn_ChartLegend, gn_ChartSymbol;
long gn_TexSpecular, gn_TexChrome, gn_TexBoing, gn_TexBubble;
wstring gs_Resource;

vector <ZGLTEXTUREX> g_mt;
vector <ZCHART_PARAM> g_Graph;
vector <wstring> g_Spinner;
BYTE g_R[34] = {0}, g_G[34] = {0}, g_B[34] = {0}, g_A[34] = {0};
long g_nUseSpinner;

long zTrace (IN wstring sPtr) {
    long nRet = 0;
    static HMODULE hDll;
    if (hDll == 0) { 
        if (sizeof(LONG_PTR) == 8) {
            hDll = LoadLibrary(L"zTrace64"); }
        else {
            hDll = LoadLibrary(L"zTrace32");
        }
    }
    if (hDll) {
        long_proc (WCHAR*);
        zProc hProc = (zProc) GetProcAddress(hDll, "zTrace");
        if (hProc) { nRet = hProc((WCHAR*) sPtr.c_str()); }
    }
    return nRet;
}

wstring EXEpath() {
    wstring sResult;
    WCHAR exepath[MAX_PATH] = {0};
    if (GetModuleFileName(0, exepath, sizeof(exepath))) {
       sResult = exepath;
       sResult = sResult.substr( 0, sResult.rfind(L"\\"));
       sResult += L"\\";
    }
    return sResult;
}

wstring STRL$(IN long N) {
    WCHAR longstr[33] = {0};
    _ltow_s(N, longstr, 10);
    return (WCHAR*) longstr;
}

HFONT zDefaultFont() {
    static HFONT hDefault;
    if (hDefault == 0) { hDefault = (HFONT) GetStockObject(ANSI_VAR_FONT); }
    return hDefault;
}

void zSetCTLFont(IN HWND hCtrl, IN HFONT hFont) {
    SendMessage(hCtrl, WM_SETFONT, (WPARAM) hFont, 0);
}

HWND zStaticCenter(IN wstring sLabel, IN long x, IN long y, IN long w, IN long h, IN HWND hParent, IN long nID) {
    HWND hCtl = CreateWindowEx(0, L"STATIC", sLabel.c_str(), WS_CHILD | WS_VISIBLE | WS_BORDER | SS_CENTER, x, y, w, h, hParent, (HMENU) nID, zInstance(), NULL);
    if (hCtl) { zSetCTLFont(hCtl, zDefaultFont()); }
    return hCtl;
}

// We need this to share UseFont among procedures
void GlobalFont(OUT ZGLFONT &UseFont, IN long RW) {
    static ZGLFONT WasFont;
    if (RW) { CopyMemory(&WasFont, &UseFont, sizeof(ZGLFONT)); }
    CopyMemory(&UseFont, &WasFont, sizeof(ZGLFONT));
}

// Main window minimum horizontal track size
long MinTrackSizeW (IN long SizeX) { // dllexport
    static long WasSizeX;
    if (SizeX) { WasSizeX = SizeX; }
    return WasSizeX;
}

// Main window minimum vertical track size
long MinTrackSizeH (IN long SizeY) { // dllexport
    static long WasSizeY;
    if (SizeY) { WasSizeY = SizeY; }
    return WasSizeY;
}

void ListDeleteAll(IN HWND hList) {
    if (hList) { SendMessage(hList, LB_RESETCONTENT, 0, 0); }
}

LRESULT ListAdd(IN HWND hList, IN WCHAR* zTmp) {
    LRESULT nIndex = 0;
    if (hList) {
       if (wcslen(zTmp)) { nIndex = SendMessage(hList, LB_ADDSTRING, 0, (LPARAM) zTmp); }
    }
    return nIndex;
}

void ListSelectPlus(IN HWND hList, IN long nIndex) {
    if (nIndex < 0) { hList = 0; }
    if (hList) {
        if (nIndex > 0) { --nIndex; } // Because ListBox is zero based
        SendMessage(hList, LB_SETCURSEL, nIndex, 0);
    }
}

LRESULT ListGetCursel(IN HWND hList) {
    LRESULT nIndex = 0;
    if (hList) {
       nIndex = SendMessage(hList, LB_GETCURSEL, 0, 0);
       if (nIndex > -1) { ++nIndex; } // Because ListBox is zero based
    }
    return nIndex;
}

wstring ListGetText (IN HWND hList, IN long nItem) {
    wstring sData = L"";
    if (nItem > 0) {
        --nItem;
        long nLen = (long) SendMessage(hList, LB_GETTEXTLEN, nItem, 0) + 1;
        WCHAR* szData = new WCHAR[nLen];
        memset(szData, 0, sizeof(WCHAR) * nLen);
        nLen = (long) SendMessage(hList, LB_GETTEXT, nItem, (LPARAM) szData);
        sData = szData;
        delete [] szData;
    }
    return sData;
}

void gl_LoadTexture(IN HWND hList) {
    if (hList == 0) { return; }

    WIN32_FIND_DATAW fd;
    WCHAR zTmp[MAX_PATH] = {0};
    WCHAR zExt[8] = {0};
    wstring sExt;

    ListDeleteAll(hList);

    wstring sImage = L".jpg.png.bmp.";
    wstring sSearch = gs_Resource; sSearch += L"Spec*.jpg";
    wcscpy_s(zTmp, (WCHAR*) sSearch.c_str());
    HANDLE hFind = FindFirstFile(zTmp, &fd);
    long nPos = 0, nCount = 0;
    if (hFind != INVALID_HANDLE_VALUE) {
       int IsOk = -1;
       while (IsOk != 0) {
           if (!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
               sSearch = fd.cFileName;
               wcscpy_s(zTmp, (WCHAR*) sSearch.c_str());
               nPos = (long) sSearch.find_last_of(L"."); // Extract extension.
               sExt = sSearch.substr(nPos); sExt += L".";
               wcscpy_s(zExt, (WCHAR*) sExt.c_str());
               CharLower(zExt);
               nPos = (long) sImage.find(zExt);
               if (nPos > -1) {
                   if (ListAdd(hList, zTmp) > -1) { nCount += 1; }
               }
           }
           if (FindNextFile(hFind, &fd) == false) { IsOk = 0; }
       }
       FindClose(hFind);
    }
    if (nCount > 0) { ListSelectPlus(hList, 1); }
}

void LoadSpinner() {
    WIN32_FIND_DATAW fd;
    WCHAR zTmp[MAX_PATH] = {0};
    wstring sSearch = gs_Resource; sSearch += L"*.ski";
    wcscpy_s(zTmp, (WCHAR*) sSearch.c_str());
    long nCount = 0;

    g_nUseSpinner = 0;

    HANDLE hFind = FindFirstFile(zTmp, &fd);
    if (hFind != INVALID_HANDLE_VALUE) {
       int IsOk = -1;
       while (IsOk != 0) {
           if (!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
               nCount +=1;
               g_Spinner.resize(nCount);
               g_Spinner[nCount - 1] = fd.cFileName;
               if (g_Spinner[nCount - 1].compare(L"bawl.ski") == 0) { g_nUseSpinner = nCount -1; }
           }
           if (FindNextFile(hFind, &fd) == false) { IsOk = 0; }
       }
       FindClose(hFind);
    }
}

void gl_InitGraph() {
    long nChartSeries = 7;
    long nChartNumber = 7;

    // The ChartSeries and ChartNumber maximum are 12 each.
    g_Chart.chartseries = min(nChartSeries, 13);
    g_Chart.chartnumber = min(nChartNumber, 13);
    g_Chart.radius = 1.0f;

    float tabZ[13] = {-24.0f,-26.0f,-28.0f,-30.0f,-34.0f,-38.0f,-42.0f,-47.0f,-50.0f,-53.0f,-57.0f,-60.0f,-62.0f};
    float tabY[13] = {-3.0f,-2.2f,-0.5f,0.6f,1.5f,2.5f,3.5f,4.0f,5.0f,6.5f,7.0f,8.3f,9.5f};
    long nChartMax = max(g_Chart.chartseries, g_Chart.chartnumber);
    long nChartMin = min(g_Chart.chartseries, g_Chart.chartnumber);

    float rChartXoff = (-g_Chart.chartseries + g_Chart.chartnumber) * 0.8333f;
    float rChartYoff = tabY[nChartMin-1];
    float rChartZoff = tabZ[nChartMax-1];
    float rChartZoom = 1.0f;

    float rChartXrot = 45.0f;
    float rChartYrot = -25.0f;

    GL_ChartSetMousing(rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot);
    gr_Percent = 0;
}

void gl_ColorInit() {
    g_R[33] = 32;  g_G[33] = 32;  g_B[33] = 32;  g_A[33] = 255;
    g_R[32] = 0;   g_G[32] = 44;  g_B[32] = 233; g_A[32] = 255;
    g_R[31] = 0;   g_G[31] = 67;  g_B[31] = 210; g_A[31] = 255;
    g_R[30] = 0;   g_G[30] = 89;  g_B[30] = 187; g_A[30] = 255;
    g_R[29] = 0;   g_G[29] = 112; g_B[29] = 164; g_A[29] = 255;
    g_R[28] = 0;   g_G[28] = 135; g_B[28] = 142; g_A[28] = 255;
    g_R[27] = 0;   g_G[27] = 159; g_B[27] = 117; g_A[27] = 255;
    g_R[26] = 0;   g_G[26] = 183; g_B[26] = 88;  g_A[26] = 255;
    g_R[25] = 0;   g_G[25] = 207; g_B[25] = 58;  g_A[25] = 255;
    g_R[24] = 0;   g_G[24] = 231; g_B[24] = 29;  g_A[24] = 255;
    g_R[23] = 26;  g_G[23] = 234; g_B[23] = 26;  g_A[23] = 255;
    g_R[22] = 52;  g_G[22] = 237; g_B[22] = 23;  g_A[22] = 255;
    g_R[21] = 79;  g_G[21] = 240; g_B[21] = 20;  g_A[21] = 255;
    g_R[20] = 105; g_G[20] = 243; g_B[20] = 17;  g_A[20] = 255;
    g_R[19] = 126; g_G[19] = 245; g_B[19] = 14;  g_A[19] = 255;
    g_R[18] = 147; g_G[18] = 248; g_B[18] = 11;  g_A[18] = 255;
    g_R[17] = 168; g_G[17] = 250; g_B[17] = 8;   g_A[17] = 255;
    g_R[16] = 189; g_G[16] = 253; g_B[16] = 5;   g_A[16] = 255;
    g_R[15] = 210; g_G[15] = 255; g_B[15] = 2;   g_A[15] = 255;
    g_R[14] = 233; g_G[14] = 255; g_B[14] = 0;   g_A[14] = 255;
    g_R[13] = 255; g_G[13] = 255; g_B[13] = 0;   g_A[13] = 255;
    g_R[12] = 255; g_G[12] = 251; g_B[12] = 0;   g_A[12] = 255;
    g_R[11] = 255; g_G[11] = 235; g_B[11] = 0;   g_A[11] = 255;
    g_R[10] = 255; g_G[10] = 215; g_B[10] = 0;   g_A[10] = 255;
    g_R[9]  = 255;  g_G[9] = 196;  g_B[9] = 0;    g_A[9] = 255;
    g_R[8]  = 255;  g_G[8] = 176;  g_B[8] = 0;    g_A[8] = 255;
    g_R[7]  = 255;  g_G[7] = 156;  g_B[7] = 0;    g_A[7] = 255;
    g_R[6]  = 253;  g_G[6] = 137;  g_B[6] = 0;    g_A[6] = 255;
    g_R[5]  = 255;  g_G[5] = 117;  g_B[5] = 0;    g_A[5] = 255;
    g_R[4]  = 255;  g_G[4] = 97;   g_B[4] = 0;    g_A[4] = 255;
    g_R[3]  = 255;  g_G[3] = 78;   g_B[3] = 0;    g_A[3] = 255;
    g_R[2]  = 255;  g_G[2] = 58;   g_B[2] = 0;    g_A[2] = 255;
    g_R[1]  = 255;  g_G[1] = 0;    g_B[1] = 0;    g_A[1] = 255;
} 

void gl_Initialize() {

    srand((unsigned) time(NULL));

    glClearColor(0.5f, 0.5f, 0.5f, 1.0f); // Grayed background, same as RGB(128,128,128)
    glClearDepth(1);
    glDepthFunc(GL_LESS);
    glEnable(GL_DEPTH_TEST);

    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST); // GL_NICEST)

    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);
    glEnable(GL_LINE_SMOOTH);
    glShadeModel(GL_SMOOTH);

    // Ambience setting.
    float LightDiffuse[4] = {0.4f, 0.4f, 0.4f, 1.0f};
    float LightAmbient[4] = {0.5f, 0.5f, 0.5f, 1.0f};
    float LightPosition[4] = {-100.0f, -100.0f, 400.0f, 1.0f};
    glLightfv(GL_LIGHT0, GL_DIFFUSE, &LightDiffuse[0]);        // Setup the Diffuse Light
    glLightfv(GL_LIGHT0, GL_AMBIENT, &LightAmbient[0]);        // Setup the Ambient Light
    glLightfv(GL_LIGHT0, GL_POSITION, &LightPosition[0]);      // Position the Light
    glMaterialfv(GL_FRONT, GL_SPECULAR, &LightDiffuse[0]);
    float rFloat = 1.0f; glMaterialfv(GL_FRONT, GL_SHININESS, &rFloat);
    glEnable(GL_LIGHT0);                                      // Enable Light ONE
    glEnable(GL_LIGHTING);                                    // Enable Lighting
    glEnable(GL_COLOR_MATERIAL);                              // Enable Coloring Of Material
    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

    glEnable(GL_TEXTURE_2D);

    long nK = 0, nI = 0, nJ = 0, nC = 0, nT = 0;

    gl_InitGraph();

    g_Chart.listindex = 1;
    glGenLists(1);

    // Draw the XYZ axis grid.
    glNewList(g_Chart.listindex, GL_COMPILE);
       glColor4ub(255, 255, 255, 64);
       GL_ChartAxis(g_Chart.chartseries * (g_Chart.radius * 2.1f) + g_Chart.radius, 10.0f, g_Chart.chartnumber * (g_Chart.radius * 2.1f) + g_Chart.radius);
    glEndList();

    g_Graph.resize((g_Chart.chartnumber + 1) * (g_Chart.chartseries + 1));

    gl_ColorInit();

    // Draw g_Chart series.
    for (nI = 1; nI <= g_Chart.chartnumber; nI++) {

        // Setup color indice
        nC = (nC + 3) % 32 + 1;

        for (nJ = 1; nJ <= g_Chart.chartseries; nJ++) {

            nK += 1;
            g_Graph[nK].nList = g_Chart.listindex + nK;
            g_Graph[nK].nColor = nC;

            g_Graph[nK].rValue = (float) (rand() % 8 + 1); // RND(1,8)

            glNewList(g_Graph[nK].nList, GL_COMPILE);

               nT = nI; if (nT > CHART_MAX) { nT = nI % CHART_MAX; }
               switch (nT) {
               case CHART_BAR:
                    g_Graph[nK].nType = nT;
                    g_Graph[nK].nTexture = gn_TexChrome;
                    g_Graph[nK].nTexSpec = TRUE;
                    GL_ChartBar(g_Chart.radius * 0.75f, g_Chart.radius * 0.75f, g_Graph[nK].rValue);
                    break;

               case CHART_CYLINDER:
                    g_Graph[nK].nType = nT;
                    g_Graph[nK].nTexture = gn_TexChrome;
                    g_Graph[nK].nTexSpec = TRUE;
                    GL_ChartCylinder(g_Chart.radius * 0.75f, g_Graph[nK].rValue, 32, 1);
                    break;

               case CHART_PYRAMIDE:
                    g_Graph[nK].nType = nT;
                    g_Graph[nK].nTexture = gn_TexChrome;
                    g_Graph[nK].nTexSpec = TRUE;
                    GL_ChartPyramide (g_Chart.radius * 0.75f, g_Graph[nK].rValue);
                    break;

               case CHART_SPHERE:
                    g_Graph[nK].nType = nT;
                    g_Graph[nK].nTexture = gn_TexBoing;
                    g_Graph[nK].nTexSpec = FALSE;
                    GL_ChartSphere(g_Chart.radius * 0.75f, g_Graph[nK].rValue, FALSE);
                    break;

               case CHART_CONE:
                    g_Graph[nK].nType = nT;
                    g_Graph[nK].nTexture = gn_TexChrome;
                    g_Graph[nK].nTexSpec = TRUE;
                    GL_ChartCone(g_Chart.radius * 0.75f, g_Graph[nK].rValue, 32, 1);
                    break;

               case CHART_POINT:
                    g_Graph[nK].nType = nT;
                    g_Graph[nK].nTexture = gn_TexBubble;
                    g_Graph[nK].nTexSpec = FALSE;
                    // ChartPoint is 2D surface, thus because of billboarding
                    // we couldn't compile it with the other 3D objects.
                    //GL_ChartPoint(g_Chart.radius * 0.75f, g_Graph[nK].rValue)
                    break;
               }

            glEndList();
        }
    }
}

void gl_DestroyTexture() {
    long K, OkDelete = 0;
    long mtCount = (long) UBOUND(g_mt);
    if (mtCount) {
       long* Texture = new long[mtCount]; memset(Texture, 0, mtCount);
       for (K = 0; K < mtCount; K++) {
           Texture[K] = g_mt[K].Texture; if (Texture[K]) { OkDelete = -1; }
       }
       if (OkDelete) { glDeleteTextures(mtCount, (GLuint*) Texture[0]); }
       delete [] Texture;
    }
}

void gl_DisableAmbience() {
    glDisable(GL_LIGHT0);        // Disable Light ONE
    glDisable(GL_LIGHTING);      // Disable Lighting
    glDisable(GL_COLOR_MATERIAL);// Disable Coloring of Material
}

void gl_EnableAmbience() {
    glEnable(GL_LIGHT0);         // Enable Light ONE
    glEnable(GL_LIGHTING);       // Enable Lighting
    glEnable(GL_COLOR_MATERIAL); // Enable Coloring of Material
}

// This is the place where to customize the charting graph.
void gl_DrawScene() {
    long nK = 0, nI = 0, nJ = 0, nC = 0;
    float rX = 0.0f, rY = 0.0f, rZ = 0.0f, 
          rHalfRadius = 0.0f, rChartXoff = 0.0f, rChartYoff = 0.0f, rChartZoff = 0.0f, rChartZoom = 0.0f, rChartXrot = 0.0f, rChartYrot = 0.0f;
    WCHAR zTitle[64] = {0};
    wstring ws;

    static long nAngle;
    nAngle +=1; nAngle = nAngle % 360;
    if (gr_Percent < 1) { gr_Percent += 0.002f; }

    static long nFlip;
    nFlip = !nFlip; if (nFlip) { return; }

    // Turn on antialiasing
    if (GL_MultiSampleARB()) { glEnable(GL_MULTISAMPLE_ARB); }

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glLoadIdentity();

    // Draw image background
    gl_DisableAmbience();
       glColor4ub(255,255,255,255);
       GL_BindTexture(GL_TEXTURE_2D, gn_TexSpecular);
       GL_DrawQuad(84.0f, 42.0f, -99.9f);
    gl_EnableAmbience();

    // Get Mousing coordinates, zooming, rotation.
    GL_ChartGetMousing(rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot);
    glTranslatef(rChartXoff, rChartYoff, rChartZoff);

    // Zooming factor.
    glScalef(rChartZoom, rChartZoom, rChartZoom);
    // Mousing rotation
    glRotatef(-rChartYrot, 1, 0, 0);
    glRotatef(-rChartXrot, 0, 1, 0);

    // Retrieve current WGL font
    ZGLFONT UseFont = {0}; WCHAR zTxt[16] = {0};
    GlobalFont(UseFont, 0);

    // Draw the chart
    glPushMatrix();

       // Use chrome texture to draw the chart graph.
       GL_BindTexture(GL_TEXTURE_2D, gn_TexChrome);
       rHalfRadius = g_Chart.radius * 0.5f;
       glTranslatef(g_Chart.radius + rHalfRadius, 0.0f, g_Chart.radius + rHalfRadius);
       nK = 0;
       for (nI = 1; nI <= g_Chart.chartnumber; nI++) {

           // Set color for the chart serie
           //ZI_glColor4f(g_Graph[nI].nColor)
           for (nJ = 1; nJ <= g_Chart.chartseries; nJ++) {
               nK += 1;
               nC = g_Graph[nK].nColor;
               glColor4ub(g_R[nC], g_G[nC], g_B[nC], g_A[nC]);
               GL_BindTexture(GL_TEXTURE_2D, g_Graph[nK].nTexture);
               GL_TexSpec(g_Graph[nK].nTexSpec);

               // Draw the matching chart item
               if (gr_Percent < 1) {

                  switch (g_Graph[nK].nType) {
                  case CHART_BAR:
                       GL_ChartBar(g_Chart.radius * 0.75f, g_Chart.radius * 0.75f, g_Graph[nK].rValue * gr_Percent);
                       break;
                  case CHART_CYLINDER:
                       GL_ChartCylinder(g_Chart.radius * 0.75f, g_Graph[nK].rValue * gr_Percent, 32, 1);
                       break;
                  case CHART_PYRAMIDE:
                       GL_ChartPyramide(g_Chart.radius * 0.75f, g_Graph[nK].rValue * gr_Percent);
                       break;
                  case CHART_SPHERE:
                       GL_ChartSphere(g_Chart.radius * 0.75f, g_Graph[nK].rValue * gr_Percent, FALSE);
                       break;
                  case CHART_CONE:
                       GL_ChartCone(g_Chart.radius * 0.75f, g_Graph[nK].rValue * gr_Percent, 32, 1);
                       break;
                  case CHART_POINT:
                       GL_ChartPoint(g_Chart.radius * 0.75f, g_Graph[nK].rValue * gr_Percent);
                       break;
                  }
               }
               else {
                  if (g_Graph[nK].nType == CHART_POINT) {
                      GL_ChartPoint(g_Chart.radius * 0.75f, g_Graph[nK].rValue); }
                  else {
                      glCallList(g_Graph[nK].nList);
                  }
               }

               GL_TexSpec(FALSE);
               glTranslatef(2.1f * g_Chart.radius, 0.0f, 0.0f);

           }
           glTranslatef((-2.1f * g_Chart.radius) * g_Chart.chartseries, 0.0f, 2.1f * g_Chart.radius);

       }

    glPopMatrix();


    // Restore the default background texture.
    GL_BindTexture(GL_TEXTURE_2D, gn_TexSpecular);

    // Draw XYZ text.
    for (nI = 0; nI < 11; nI++) {
        wcscpy_s(zTxt, (WCHAR*) STRL$(nI * 10).c_str());
        GL_DrawText (gn_GLwnd, UseFont, g_Chart.chartseries * (g_Chart.radius * 2.1f) + g_Chart.radius - 0.1f, (float) nI, 0, zTxt, ZD_ARGB(255, 0,255,0));
        GL_DrawText (gn_GLwnd, UseFont, 0.1f, (float) nI, g_Chart.chartnumber * (g_Chart.radius * 2.1f) + g_Chart.radius + 0.5f, zTxt, ZD_ARGB(255, 0,255,0));
    }

    // Draw the XYZ grid
    glLineWidth(1.0f);
    glCallList(g_Chart.listindex);

    // Draw the podium
    glPushMatrix();
       GL_TexSpec(TRUE);
          glColor4ub(255,255,255, 128);
          rX = g_Chart.chartseries * (g_Chart.radius * 1.05f);
          rY = g_Chart.chartnumber * (g_Chart.radius * 1.05f);
          rZ = 0;
          glTranslatef(rX + g_Chart.radius * 0.5f, -0.1f, rY + g_Chart.radius * 0.5f);
          GL_ChartBar(rX + g_Chart.radius * 1.5f, rY + g_Chart.radius * 1.5f, -0.2f);
       GL_TexSpec(FALSE);
    glPopMatrix();

    // Draw XYZ translucent glass panels
    GL_TexSpec(TRUE);
       rX = g_Chart.chartseries * (g_Chart.radius * 2.1f) + g_Chart.radius;
       rY = 10;
       rZ = g_Chart.chartnumber * (g_Chart.radius * 2.1f) + g_Chart.radius;
       glColor4ub(240,240,255, 64); // Transparent light blue color.
       GL_ChartAxisPanel(rX, rY, rZ);
    GL_TexSpec(FALSE);

    // Draw legend using GDImage bitmapped font.
    glPushMatrix();
       gl_DisableAmbience();
       glColor4ub(255,255,255,255);
       for (nI = 1; nI <= g_Chart.chartnumber; nI++) {
           glPushMatrix();
              glTranslatef((2.1f * g_Chart.radius) * g_Chart.chartseries + g_Chart.radius, rHalfRadius * 0.6f, g_Chart.radius + rHalfRadius * 1.25f);

              // Billboard to keep it in front of the camera.
              GL_BillBoard();
              ws = (WCHAR) 128 + (WCHAR) nI;
              GL_ChartSymbol(gn_GLwnd, gn_ChartSymbol, 0, 0, 0, (WCHAR*) ws.c_str());

           glPopMatrix();
           glTranslatef(0, 0, 2.1f * g_Chart.radius);
       }
       gl_EnableAmbience();
    glPopMatrix();

    // Draw small letters in front of each chart bar.
    glPushMatrix();
       gl_DisableAmbience();
       glColor4ub(255,255,255, 255);
       glTranslatef(g_Chart.radius + rHalfRadius * 0.5f, 0.0f, 2.1f * g_Chart.radius * g_Chart.chartnumber + rHalfRadius);
       for (nJ = 1; nJ <= g_Chart.chartseries; nJ++) {
           ws = (WCHAR) 64 + (WCHAR) nJ;
           GL_ChartLabel(gn_GLwnd, gn_ChartLegend, 0, 0, 0, (WCHAR*) ws.c_str());
           glTranslatef(2.1f * g_Chart.radius, 0.0f, 0.0f);
       }
       glTranslatef((-2.1f * g_Chart.radius) * g_Chart.chartseries, 0.0f, 2.1f * g_Chart.radius);
       gl_EnableAmbience();
    glPopMatrix();

    // Draw label using GDImage bitmapped font.
    glPushMatrix();
       glTranslatef(9.2f, 0.0f, 2.1f * g_Chart.radius * g_Chart.chartnumber + rHalfRadius * 3.0f + 0.35f);
       glRotatef(-90, 1, 0, 0);
       glColor4ub(255,255,0, 255);
       ws = L"GDImage "; ws += ZI_Version();
       GL_ChartLabel(gn_GLwnd, gn_ChartCaption, 0, 0, 0, (WCHAR*) ws.c_str());
    glPopMatrix();

    // Draw the chart caption using GDImage bitmapped font.
    glPushMatrix();
       glLoadIdentity();
       gl_DisableAmbience();
       glDisable(GL_DEPTH_TEST); // We want to keep it in front.
       wcscpy_s(zTitle, L"The title of the chart");
       glTranslatef(GL_ChartGetTextLength(gn_ChartCaption, zTitle) * -0.5f, -8.0f, -20.0f);
       glColor4ub(255,0,0, 255);
       GL_ChartLabel(gn_GLwnd, gn_ChartCaption, 0, 0, 0, zTitle);
       glEnable(GL_DEPTH_TEST); // Don't forget to re-enable DEPTH_TEST
       gl_EnableAmbience();
    glPopMatrix();

    // Draw zoom percentage
    ws = L"Zoom "; ws += STRL$((long) (rChartZoom * 100.0f)); ws += L" %";
    ZI_DrawGLText(gn_GLwnd, UseFont, 10, 10 + UseFont.fontHeight * 0, (WCHAR*) ws.c_str(), ZD_ARGB(255, 255,255,0));

    glFlush();

    // Turn off antialiasing
    if (GL_MultiSampleARB()) { glDisable(GL_MULTISAMPLE_ARB); }

    // Update display
    ZI_UpdateGLWindow(gn_GLwnd);

}

LRESULT CALLBACK WndProc(IN HWND hWin, IN UINT uMsg, IN WPARAM wParam, IN LPARAM lParam) {

    HWND hCtrl;
    long wmId, wmEvent;
    MINMAXINFO* pMM;
    wstring ws;

    switch (uMsg) {

    case WM_GETMINMAXINFO:
         pMM = (MINMAXINFO*) lParam;
         pMM->ptMinTrackSize.x = MinTrackSizeW(0);
         pMM->ptMinTrackSize.y = MinTrackSizeH(0);
         break;

    case WM_TIMER:
         gl_DrawScene();
         return 0;

    case WM_MOUSEWHEEL:
         if (GetFocus() != gn_GLwnd) { SetFocus(gn_GLwnd); }
         break;

    case WM_SIZE:
         ZI_ResizeGLWindow(gn_GLwnd);
         gl_DrawScene();
         break;

    case WM_COMMAND:
         wmId    = LOWORD(wParam);
         wmEvent = HIWORD(wParam);

         switch (wmId) {

         case IDC_RESET:
              gl_InitGraph();
              ZI_ResizeGLWindow(gn_GLwnd);
              gl_DrawScene();
              break;

         case IDC_LISTBOX:
              if (wmEvent == LBN_SELCHANGE) {
                  hCtrl = GetDlgItem(hWin, IDC_LISTBOX);
                  ws = gs_Resource; ws += ListGetText(hCtrl, (long) ListGetCursel(hCtrl));
                  wcscpy_s(g_mt[0].FullName, (WCHAR*) ws.c_str());
                  ZI_UpdateNamedGLTextureFromFileEx(g_mt[0].FullName, g_mt[0].Texture, g_mt[0].Square);
              }
              break;

         }
         return 0;

    case WM_DESTROY:
         PostQuitMessage(0);
         return 0;
    }
    return DefWindowProc(hWin, uMsg, wParam, lParam);
}

void Animate (IN LPVOID nDelay) {
    DWORD nWait = 0;
    long nFrame;
    for (;;) {
        nFrame = ZD_GetObjectFrameToUse(ID_SPIN01) + 1; if (nFrame > ZD_GetObjectFrameCount(ID_SPIN01)) { nFrame = 1; }
        ZD_SetObjectFrameToUse(ID_SPIN01, nFrame, TRUE);
        Sleep((long) nDelay);
    }
}

long StartAnimation (IN long Delay) {
    long nRet = LB_ERR;
    DWORD dwThreadId = 0;
    HANDLE hThread = CreateThread(NULL,                                  // default security attributes
                                  0,                                     // use default stack size  
                                  (LPTHREAD_START_ROUTINE ) Animate,     // thread function name
                                  (LPVOID) Delay,                        // argument to thread function 
                                  0,                                     // use default creation flags 
                                  &dwThreadId);                          // returns the thread identifier
    if (hThread) { nRet = 0; Sleep(100); }
    CloseHandle(hThread);
    return nRet;
}

LRESULT CALLBACK MyCallBack(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    LRESULT nRet = 0; // Do not stop the event processing in GDImage
    static long nCount;

    HWND hParent = GetParent(hWnd);
   
    if (hWnd == GetDlgItem(hParent, IDC_SPINNER)) { // In case we use the same callback for several GDImage control
                                                    // make sure that we handle the good one.
       long nReserved = 0, imgW = 0, imgH = 0;
       HBITMAP hBitmap;
       wstring ws = gs_Resource;

       switch (uMsg) {

       case WM_LBUTTONDOWN:
       case WM_LBUTTONDBLCLK:
            g_nUseSpinner -= 1; if (g_nUseSpinner < 0) { g_nUseSpinner = (long) g_Spinner.size() - 1; }
            ws.append( g_Spinner[g_nUseSpinner]);
            hBitmap = skSkiToDib ((WCHAR*) ws.c_str(), nReserved);
            ZI_GetBitmapSize(hBitmap, imgW, imgH);
            ZD_DrawBitmapToCtrl(hWnd, (128 - imgH) / 2, (128 - imgH) / 2, hBitmap, ZD_ColorARGB(255, 0), ID_SPIN01, ZS_VISIBLE);
            ZD_SetObjectFrameCount(ID_SPIN01, (BYTE) (imgW / imgH));
            ZD_SetObjectFrameToUse(ID_SPIN01, 1, FALSE);
            break;

       case WM_RBUTTONDOWN:
       case WM_RBUTTONDBLCLK:
            g_nUseSpinner += 1; if (g_nUseSpinner > g_Spinner.size() - 1) { g_nUseSpinner = 0; }
            ws.append( g_Spinner[g_nUseSpinner]);
            hBitmap = skSkiToDib ((WCHAR*) ws.c_str(), nReserved);
            ZI_GetBitmapSize(hBitmap, imgW, imgH);
            ZD_DrawBitmapToCtrl(hWnd, (128 - imgH) / 2, (128 - imgH) / 2, hBitmap, ZD_ColorARGB(255, 0), ID_SPIN01, ZS_VISIBLE);
            ZD_SetObjectFrameCount(ID_SPIN01, (BYTE) (imgW / imgH));
            ZD_SetObjectFrameToUse(ID_SPIN01, 1, FALSE);
            break;

       }
    }

    return nRet;
}

int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow) {

    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);

    MSG msg;
    long nRet = 0;
    WNDCLASSEX wcx = {0};
    WCHAR szWindowClass[] = L"ZCHART3D";

    RECT rc;
    long x, y, IsInitialized;
    DWORD dwStyle, dwExStyle;
    HWND hMain, hList, hFound, hCtrl;

    ZGLFONT UseFont = {0};

    HANDLE hMutex = CreateMutex(NULL, 0, szWindowClass);
    if (hMutex) {
       if (GetLastError() == ERROR_ALREADY_EXISTS) {
          for(;;){
              hFound = FindWindow(szWindowClass, NULL); if (hFound) { break; }
              while (PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE)) { }
          }
          if (IsIconic(hFound)) { ShowWindow(hFound, SW_RESTORE); }
          SetForegroundWindow(hFound);
          return 0;
       }
    }

    wcx.cbSize = sizeof(wcx);
    IsInitialized = GetClassInfoEx(hInstance, szWindowClass, &wcx);
    if (!IsInitialized) {
       wcx.style         = 0; // CS_HREDRAW | CS_VREDRAW;
       wcx.lpfnWndProc   = WndProc;
       wcx.cbClsExtra    = 0;
       wcx.cbWndExtra    = 0;
       wcx.hInstance     = hInstance;
       wcx.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_APPLICATION));
       wcx.hCursor       = LoadCursor(NULL, IDC_ARROW);
       wcx.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
       wcx.lpszMenuName  = NULL;
       wcx.lpszClassName = szWindowClass;
       wcx.hIconSm       = wcx.hIcon;
       if (RegisterClassEx(&wcx)) { IsInitialized = -1; }
    }
    if (IsInitialized) {

       // Window Extended Style
       dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
       // Windows Style
       dwStyle = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
       SetRect(&rc, 0, 0, ClientW, ClientH);
       AdjustWindowRectEx(&rc, dwStyle, FALSE, dwExStyle);    // Adjust Window To True Requested Size

       x = max((GetSystemMetrics(SM_CXSCREEN) - rc.right - rc.left) / 2, 0);
       y = max((GetSystemMetrics(SM_CYSCREEN) - rc.bottom - rc.top) / 2, 0);
       MinTrackSizeW(rc.right - rc.left);
       MinTrackSizeH(rc.bottom - rc.top);

       // Create The Window
       wstring sCaption = L"3D Chart demo";
       if (sizeof(LONG_PTR) == 8) {
           sCaption += L" 64-bit"; }
       else {
           sCaption += L" 32-bit";
       }

       hMain = CreateWindowEx(dwExStyle,                  // Extended Style For The Window
                             szWindowClass,               // Class Name
                             (WCHAR*) sCaption.c_str(),   // Window Title
                             dwStyle,                     // Defined Window Style
                             x, y,                        // Window Position
                             MinTrackSizeW(0),            // Calculate Window Width 
                             MinTrackSizeH(0),            // Calculate Window Height
                             NULL,                        // No Parent Window
                             NULL,                        // No Menu
                             hInstance,                   // Instance
                             NULL);                       // Dont Pass Anything To WM_CREATE

       if (hMain) {

          gs_Resource = EXEpath(); gs_Resource.append(L"Resource\\");

          // Create button "Reset"
          CreateWindowEx(0, L"BUTTON", L"Reset", WS_CHILD | WS_VISIBLE | WS_TABSTOP, 
                             ClientW - (8 + 120 + 8), 10 + (22 + 5) * 2 - 2, 120, 30, hMain, (HMENU) IDC_RESET, hInstance, NULL);

          // Create Static.
          zStaticCenter(L"\n\rLeft mouse button:\n\rchange view angle.\n\r\n\rRight mouse button:\n\rmove graph location.\n\r\n\rMouse wheel zoom.", 
                        ClientW - (8 + 120 + 8), 10 + (22 + 5) * 5 + 5, 120, 120, hMain, IDC_STATIC);

          // Create ListBox to select texture from.
          hList = CreateWindowEx(0, L"ListBox", NULL, WS_CHILD | WS_BORDER | WS_VISIBLE | WS_VSCROLL | LBS_SORT | LBS_NOTIFY | LBS_HASSTRINGS | LBS_DISABLENOSCROLL,
                                 ClientW - (8 + 120 + 8), 10 + (22 + 5) * 10 + 5, 120, 120, hMain, (HMENU) IDC_LISTBOX, hInstance, NULL);
          zSetCTLFont(hList, zDefaultFont());
          gl_LoadTexture(hList);

          // *******************************************************************************
          // Alternate methode to create a GDImage OpenGL control
          // Note: when GDImage is active the OpenGL $GLImageClassName is already registered
          // -------------------------------------------------------------------------------
          // 6.04 Create antialiased control.
          dwStyle = WS_CHILD | WS_VISIBLE;
          dwExStyle = 0;
          gn_GLwnd = WGL_CreateWindow(dwExStyle, dwStyle, 0, 0, 844, ClientH, hMain, (HMENU) IDC_CTRL);

          if (gn_GLwnd) {
             // Anchor the GDImage WGL control (make it a resizable)
             ZI_SetAnchorMode(GetDlgItem(hMain, IDC_CTRL), ANCHOR_HEIGHT_WIDTH);

             // Use GDImage OpenGL automatic mousing support.
             GL_UseMouseManager(gn_GLwnd);

             // Create the textures.
             g_mt.resize(4);
             wstring ws = gs_Resource; ws += ListGetText(hList, (long) ListGetCursel(hList));
             wcscpy_s(g_mt[0].FullName, (WCHAR*) ws.c_str()); g_mt[0].ID = 1; g_mt[0].Square = 0;
             ws = gs_Resource; ws += L"chrome.jpg"; wcscpy_s(g_mt[1].FullName, (WCHAR*) ws.c_str()); g_mt[1].ID = 2; g_mt[1].Square = 0;
             ws = gs_Resource; ws += L"boing.png";  wcscpy_s(g_mt[2].FullName, (WCHAR*) ws.c_str()); g_mt[2].ID = 3; g_mt[2].Square = 0;
             ws = gs_Resource; ws += L"bubble.png"; wcscpy_s(g_mt[3].FullName, (WCHAR*) ws.c_str()); g_mt[3].ID = 4; g_mt[3].Square = 0;
             ZI_MakeMultipleTexture(&g_mt[0], (long) UBOUND(g_mt));
             gn_TexSpecular = g_mt[0].Texture;
             gn_TexChrome   = g_mt[1].Texture;
             gn_TexBoing    = g_mt[2].Texture;
             gn_TexBubble   = g_mt[3].Texture;

             gl_Initialize();

             // Create a WGL font for the XYZ axis.
             ZI_BuildGLfont(ZI_GetGLDC(gn_GLwnd), UseFont);  // Build OpenGL font for our OpenGL window
             GlobalFont(UseFont, 1);

             // Create a GDImage bitmapped font for the chart labels.
             // Valid font size are: 16,32,64
//           gn_ChartCaption = GL_CreateFontTexture(gn_GLwnd, L"TREBUCHET MS", 32, 0xFFFFFFFF);
             ws = gs_Resource; ws.append(L"HomeRem.ttf");
             gn_ChartCaption = GL_CreateFontTexture(gn_GLwnd, (WCHAR*) ws.c_str(), 32, 0xFFFFFFFF);
             gn_ChartSymbol  = GL_CreateFontTexture(gn_GLwnd, L"WINGDINGS", 32, 0xFFFFFFFF);
             gn_ChartLegend  = GL_CreateFontTexture(gn_GLwnd, L"TREBUCHET MS", 16, 0xFFFFFFFF);


             wstring SkinTheme = EXEpath(); SkinTheme += L"Onyx.sks";
             if (skInitEngine ((WCHAR*) SkinTheme.c_str(), L"")) {
                 skSkinWindow(hMain, L"Dock|Undock|Minimize|Maximize|Restore|Close");
                 // Anchor child controls after skinning of the main window, using WinLIFT.
                 skSetAnchorCtrl(GetDlgItem(hMain, IDC_RESET), ANCHOR_RIGHT);
                 skSetAnchorCtrl(GetDlgItem(hMain, IDC_STATIC), ANCHOR_RIGHT);
                 skSetAnchorCtrl(GetDlgItem(hMain, IDC_LISTBOX), ANCHOR_RIGHT); 

                 // For the fun we add a little GDImage animation
                 hCtrl = CreateWindowEx(0, GDImageClassName, NULL,
                                         WS_CHILD | WS_VISIBLE,
                                         ClientW - (128 + 12), ClientH - (128 + 12), 128, 128,
                                         hMain, (HMENU) IDC_SPINNER,
                                         hInstance, NULL);

                 long imgW = 0, imgH= 0, nReserved = 0;
                 LoadSpinner();
                 ws = gs_Resource; ws.append(g_Spinner[g_nUseSpinner]);
                 HBITMAP hBitmap = skSkiToDib ((WCHAR*) ws.c_str(), nReserved);
                 ZI_GetBitmapSize(hBitmap, imgW, imgH);
                 ZD_DrawBitmapToCtrl(hCtrl, (128 - imgH) / 2, (128 - imgH) / 2, hBitmap, ZD_ColorARGB(255, 0), ID_SPIN01, ZS_VISIBLE);
                 // Use the WinLIFT background for better illusion.
                 ZI_UseWinLIFTbackground(hCtrl, TRUE, TRUE);
                 ZD_SetObjectFrameCount(ID_SPIN01, (BYTE) (imgW / imgH));
                 ZD_SetObjectFrameToUse(ID_SPIN01, 1, FALSE);
                 skSetAnchorCtrl(GetDlgItem(hMain, IDC_SPINNER), ANCHOR_BOTTOM_RIGHT); 
                 // Lock and disable the sprite image.
                 ZD_SetObjectLocked(ID_SPIN01, TRUE);

                 // We use a callback to monitor the GDImage spinner control messages
                 // Create a WM_LBUTTONDOWN event
                 ZI_EventMessage((LONG_PTR) MyCallBack, WM_LBUTTONDOWN, TRUE);
                 ZI_EventMessage((LONG_PTR) MyCallBack, WM_LBUTTONDBLCLK, TRUE);
                 // Create a WM_RBUTTONDOWN event
                 ZI_EventMessage((LONG_PTR) MyCallBack, WM_RBUTTONDOWN, TRUE);
                 ZI_EventMessage((LONG_PTR) MyCallBack, WM_RBUTTONDBLCLK, TRUE);

                 }
             else {
                 // Anchor child controls if skinning has failed, using GDImage.
                 ZI_SetAnchorMode(GetDlgItem(hMain, IDC_RESET), ANCHOR_RIGHT);
                 ZI_SetAnchorMode(GetDlgItem(hMain, IDC_STATIC), ANCHOR_RIGHT);
                 ZI_SetAnchorMode(GetDlgItem(hMain, IDC_LISTBOX), ANCHOR_RIGHT);
             }

             // Show the main window
             ShowWindow(hMain, nCmdShow);
             SetForegroundWindow(hMain);                  // Slightly Higher Priority
             SetFocus(hMain);                             // Sets Keyboard Focus To The Window

             // Main message pump.
             SetTimer(hMain, 1, 0, 0);   // 66 FPS
             StartAnimation(33);
             while (GetMessage(&msg, NULL, 0, 0)) {
                   TranslateMessage(&msg);
                   DispatchMessage(&msg);
             }
             nRet = (long) msg.wParam;
             KillTimer(hMain, 1);

             // Cleanup texture from memory
             gl_DestroyTexture();
             // Delete GLFont.
             ZI_DeleteGLFont(UseFont);

          }
       }
    }

    if (hMutex) { CloseHandle(hMutex); }
    return nRet;
}
